/*
 * This is Tampa Ping-Pong convers/conversd derived from the wampes
 * convers package written by Dieter Deyke <deyke@mdddhd.fc.hp.com>
 * Modifications by Fred Baumgarten <dc6iq@insu1.etec.uni-karlsruhe.de>
 *
 * Modifications by Brian A. Lantz/KO4KS <brian@lantz.com>
 * $Revision: 1.14 $$Date: 1995/06/16 14:47:52 $
 */

#include <time.h>

#ifndef SOMAXCONN
#define SOMAXCONN       5
#endif

#ifndef WNOHANG
#define WNOHANG         1
#endif

#ifdef POSIX
#define FD_SET_TYPE fd_set
#endif

#ifndef FD_SET_TYPE
#define FD_SET_TYPE struct fd_set
#endif

#if defined(__TURBOC__) || defined(__STDC__)
#define __ARGS(x)       x
#ifndef __DOTS
#define __DOTS		,...
#endif
#else
#define __ARGS(x)       ()
#define const
#ifndef __DOTS
#define __DOTS
#endif
#endif

#ifndef min
#define min(a,b) ((a) >= (b) ? (b) : (a))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#endif

#define MAXCHANNEL      32767
#define uchar(x) ((x) & 0xff)

struct mbuf {
  struct mbuf *next;
  char *data;
};

#define NULLCHAR ((char *) 0)

struct clist {
  int channel;			/* channel number */
  int channelop;		/* channel operator status */
  long time;			/* join time */
  struct clist *next;		/* pointer to next entry */
};

#define NULLCLIST ((struct clist *) 0)

#define NAMESIZE (64 - 1)
#define PERSSIZE (512 - 1)
#define AWAYSIZE (512 - 1)
#define NOTIFYSIZE (512 - 1)

struct connection {
  int type;                     /* Connection type */
#define CT_UNKNOWN      0
#define CT_USER         1
#define CT_HOST         2
#define CT_CLOSED       3
#define CT_OBSERVER	4
  char name[NAMESIZE + 1];      /* Name of user or host */
  char host[NAMESIZE + 1];      /* Name of host where user is logged on */
  char rev[NAMESIZE + 1];       /* revision of software (CT_HOST) */
  char nickname[NAMESIZE + 1];	/* Nickname of user */
  int  features;
#define FEATURE_AWAY	1	/* a - "away feature" */
#define FEATURE_FWD	2	/* d - "destination forwarding" */
#define FEATURE_MODES	4	/* m - "channel modes" */
#define FEATURE_LINK	8	/* p - "ping pong link measurement" */
#define FEATURE_UDAT	16	/* u - "udat command extension and user command understood both" */
#define FEATURE_NICK	32	/* n - "TNOS Nickname extensions" */
  struct connection *via;       /* Pointer to neighbor host */
  int channel;                  /* current channel number */
  struct clist *chan_list;	/* linked list of joined channels */
  long time;                    /* Connect time */
  int locked;                   /* Set if mesg already sent */
  int fd;                       /* Socket descriptor */
  char ibuf[2048];              /* Input buffer */
  int icnt;                     /* Number of bytes in input buffer */
  struct mbuf *obuf;            /* Output queue */
  long received;                /* Number of bytes received */
  long xmitted;                 /* Number of bytes transmitted */
  char away[AWAYSIZE + 1];      /* Away string */
  int amprnet;			/* is it /..UDAT (1) or /..USER (0) ? */
  int oldaway;			/* is it the pre3.02 Version of away ? */
  long atime;                   /* time of last "away" state change */
  long mtime;                   /* time of last message receive */
  char pers[PERSSIZE + 1];      /* Personal string */
  int verbose;                  /* verbose mode */
  char prompt[4];               /* prompt mode */
  char query[NAMESIZE + 1];     /* name of queried user */
  char notify[NOTIFYSIZE + 1];  /* List of calls you like to be notified */
  int charset_in;               /* charset - default ISO (ansi) */
  int charset_out;              /* charset - default ISO (ansi) */
  int channelop;		/* channel operator of current channel ? */
  int operator;			/* convers operator */
  int expected;			/* expected answer */
  int invitation_channel;	/* last invitation was from this channel */
  int width;			/* user screen width */
  int observer;
  struct connection *next;      /* Linked list pointer */
};

#define CM_UNKNOWN   (1 << CT_UNKNOWN)
#define CM_USER      (1 << CT_USER)
#define CM_HOST      (1 << CT_HOST)
#define CM_CLOSED    (1 << CT_CLOSED)
#define CM_OBSERVER  (1 << CT_OBSERVER)

#define NULLCONNECTION  ((struct connection *) 0)

#define DEF_NR_PERMLINKS 20

#define HOSTNAMESIZE (80 - 1)
#define PCMDSIZE (256 - 1)

struct permlink {
  char name[HOSTNAMESIZE + 1];  /* Name of host */
  char socket[HOSTNAMESIZE + 1];/* Name of socket to connect to */
  char command[PCMDSIZE + 1];   /* Optional connect command */
  struct connection *connection;/* Pointer to associated connection */
  struct permlink *primary;	/* If !NULL, points to link we backup for */
  struct permlink *backup;	/* If !NULL, points to our backup link*/
  long statetime;               /* Time of last (dis)connect */
  int tries;                    /* Number of connect tries */
  long waittime;                /* Time between connect tries */
  long retrytime;               /* Time of next connect try */
  long testwaittime;            /* Time between tries */
  long testnexttime;            /* Time of next test */
  long rxtime;                  /* rtt by other side */
  long txtime;                  /* rtt found out by me */
  int quality;			/* quality of given link */
  int group;			/* link list group */
  int permanent;		/* this is a permanent entered link */
};

#define NULLPERMLINK  ((struct permlink *) 0)
#define NULLPERMARRAY  ((struct permlink **) 0)


extern struct permlink **permarray;

struct destination {
  char name[HOSTNAMESIZE + 1];	/* destination name */
  char rev[NAMESIZE + 1];       /* revision of software (CT_HOST) */
  struct permlink *link;	/* link to this destination */
  long rtt;			/* round trip time to this host */
  long last_sent_rtt;		/* last donwnstream sent rtt */
  int connected;		/* is it connected now? (used by notify_destinations) */
  int downstream[DEF_NR_PERMLINKS];	/* all up/downstream times */
  struct destination *next;	/* a one dimansional list is ok for now :-) */
};

#define NULLDESTINATION  ((struct destination *) 0)

struct channel {
  char name[80];		/* ascii name of channel */
#define TOPICSIZE (512 - 1)
  char topic[TOPICSIZE + 1];	/* topic of channel */
  long time;			/* when was the topic set ? */
  int chan;			/* integer value of channel (<32768 !) */
  int flags;			/* channel flags */
#define M_CHAN_S 0x01	/* secret channel - number invisible */
#define M_CHAN_P 0x02	/* private channel - only join by invitation */
#define M_CHAN_T 0x04	/* topic settable by operator only */
#define M_CHAN_I 0x08	/* invisible channel - only sysops can see */
#define M_CHAN_M 0x10	/* moderated channel - operators may talk */
#define M_CHAN_L 0x20	/* local channel - no forwarding */
  struct channel *next;		/* linked list again */
};

#define NULLCHANNEL  ((struct channel *) 0)

struct cmdtable {
  char *name;
  void (*fnc)();
  int states;
};

extern struct cmdtable cmdtable[];


struct extendedcmds {
  char *name;
  struct extendedcmds *next;
};

#define NULLEXTCMD ((struct extendedcmds *)0)
extern struct extendedcmds *ecmds;


struct stats	{
	long	rx;
	long	tx;
	time_t	start;
};

#define ONE_HOUR (60*60)
#define ONE_DAY (60*60*24)
#define ONE_K (1024L)
#define ONE_MEG (1024L * 1024L)


#define LEV_ALWAYS	-1	/* always log */
#define LEV_NOTHING	0	/* log nothing at all */
#define LEV_WARNING	1	/* log critical events, startup, etc. */
#define LEV_CONFIG	2	/* plus log config interpretation */
#define LEV_LINKS	3	/* plus log significant events, link up/down, etc. */
#define LEV_CMDS	4	/* plus log cmd changes, channel creation, mode changes, etc. */
#define LEV_USERS	5	/* plus log user's entry/leaving/channel change - network wide */
#define LEV_HOSTCMDS	6	/* plus log all HOST commands received */


extern void appendstring __ARGS((struct connection *cp, const char *string));
extern void appendc __ARGS((struct connection *cp, const int n, const int ast));
extern void appendprompt __ARGS((struct connection *cp, const int ast));
extern void destroy_channel __ARGS((int number));
extern void free_connection __ARGS((struct connection *cp));
extern int queuelength __ARGS((const struct mbuf *bp));
extern void free_closed_connections __ARGS((void));
extern char *getarg __ARGS((char *line, int all));
extern char *getargcs __ARGS((char *line, int all));
extern void getTXname __ARGS((struct connection *cp, char *buf));
extern char *formatline __ARGS((char *prefix, char *text, int linelen));
extern void notify_destinations __ARGS((struct destination *dest,const char *host, const char *rev, int state));
extern char *ts __ARGS((long gmt));
extern char *ts2 __ARGS((long gmt));
extern char *ts3 __ARGS((long seconds, char *buffer));
extern char *ts4 __ARGS((long seconds));
extern struct connection *alloc_connection __ARGS((int fd, int linuxbug));
extern void accept_connect_request __ARGS((int flisten));
extern void clear_locks __ARGS((void));
extern int count_user __ARGS((int channel));
extern int count_topics __ARGS((void));
#if defined(oldAWAY)
extern void send_awaymsg __ARGS((char *name, char *host, int channel, long time, char *text));
#else
extern void send_awaymsg __ARGS((char *name, char *host, long time, char *text));
#endif
extern void send_mode __ARGS((struct channel *chan));
extern void send_opermsg __ARGS((char *name, char *host, char *from, int channel));
extern void send_persmsg __ARGS((char *name, char *host, int channel, char *text, long time));
extern void send_topic __ARGS((char *name, char *host, long time, int channel, char *text));
extern void send_user_change_msg __ARGS((char *name, char *host, int oldchannel, int newchannel, char *pers, long time, int observer));
extern void send_msg_to_user __ARGS((char *fromname, char *toname, char *text));
extern void send_msg_to_channel __ARGS((char *fromname, int channel, char *text));
extern void send_invite_msg __ARGS((char *fromname, char *toname, int channel));
extern void log_user_change __ARGS((char *name, char *host, int oldchannel, int newchannel));
extern void update_destinations __ARGS((struct permlink *p, char *name, long rtt, char *rev));
extern struct permlink *update_permlinks __ARGS((char *name, struct connection *cp, int isperm));
extern void connect_permlinks __ARGS((void));
extern int get_hash __ARGS((char *name));
extern int ecmd_exists __ARGS((char *cmdname));
extern void update_ecmds __ARGS((void));
extern void update_stats __ARGS((time_t now));
extern void init_stats __ARGS((void));
extern void mysighup __ARGS((int sig));
extern int dohelper __ARGS((char *name, const char *search));
extern void logit __ARGS((int level, const char *fmt,...));

extern struct connection *connections;
extern struct permlink *permlinks;
extern struct permlink *notpermlinks;
extern struct destination *destinations;
extern struct channel *channels;
extern struct stats daily[];
extern struct stats hourly[];
extern time_t starttime;
extern time_t nextdailystats;
extern time_t nexthourlystats;
extern long hoursonline;
extern long daysonline;

extern long boottime;
extern long currtime;
extern char *myhostname;
extern char *mysysinfo;
extern char *myemailaddr;
extern char userfile[];
extern char motdfile[];
extern char restrictfile[];
extern char accessfile[];
extern char noaccessfile[];
extern char helpfile[];
extern char observerfile[];
extern char convtype[];
extern char myrev[];
extern char myfeatures[];
extern int amprnet;
extern int secretnumber;
extern int restrictedmode;
extern char OBSERVERID[];
extern int NR_PERMLINKS;

#if defined (__OS2__)
#define strcasecmp stricmp
#define strncasecmp strnicmp
#endif
 
